import 'package:flutter/material.dart';
import 'package:verttice_logic/verttice_logic.dart';
import 'form_validators.dart';

class FormItem {
  final FormFF field;

  FormItem({required this.field});
}

abstract class FormFF {
  final String name;
  final String displayName;
  final bool showOnTable;
  final bool showOnForm;
  final List<String>? pathToField;
  final dynamic Function(Map<String, dynamic> rowData)? dataBuilder;
  final ValidatorFF? validator;
  final dynamic defaultValue;
  final FilterTypesFF filterType = FilterTypesFF.none;
  final double tableColumnWidth;
  const FormFF({
    required this.name,
    required this.displayName,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.dataBuilder,
    this.validator,
    this.defaultValue,
    this.tableColumnWidth = 100,
  });
}

enum SelectorColors {
  green,
  red,
  orange,
  yellow,
  grey,
}

class ColorSelectorFF implements FormFF {
  @override
  final String name;
  @override
  final String displayName;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;

  @override
  final String? defaultValue;
  @override
  final SelectorColors Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final ValidatorFF? validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  final List<SelectorColors> colors;

  ColorSelectorFF({
    required this.name,
    required this.displayName,
    required this.colors,
    this.pathToField,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class RichTextFF implements FormFF {
  @override
  final String name;
  @override
  final String displayName;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;

  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  RichTextFF({
    required this.name,
    required this.displayName,
    this.pathToField,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 160,
  });
}

class TextFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;

  /// in order to format a string like this GHJ45GHJHN to GHJ-45-GHJHN use String like this: "###-00-#####"
  final String? textFormatter;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final StringValidatorFF validator;
  @override
  final double tableColumnWidth;

  TextFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.textFormatter,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class TextBoxFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  TextBoxFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 160,
  });
}

class TextListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final List<String>? defaultValue;
  @override
  final List<String> Function(Map<String, dynamic> rowData)? dataBuilder;

  @override
  final BaseStringListValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;

  TextListFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.defaultValue,
    this.showOnForm = true,
    this.validator = const StringListOrNullValidatorFF(),
    this.tableColumnWidth = 120,
  });
}

class SelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  final List<dynamic> options;
  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  SelectFF({
    required this.displayName,
    required this.name,
    required this.options,
    this.defaultValue,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.tableColumnWidth = 100,
    this.validator = const StringValidatorFF(),
  });
}

class GooglePlacesFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BaseMapValidatorFF validator;

  @override
  final double tableColumnWidth;

  @override
  final FilterTypesFF filterType = FilterTypesFF.string;
  GooglePlacesFF({
    required this.displayName,
    required this.name,
    this.defaultValue,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const MapOrNullValidatorFF(mapValidator: {}),
    this.tableColumnWidth = 200,
  });
}

class NumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final List<String>? pathToField;

  final String Function(double? value)? tableRenderAsString;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final double? defaultValue;
  @override
  final double Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final NumberValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.number;

  @override
  final double tableColumnWidth;

  NumberFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.tableRenderAsString,
    this.defaultValue,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const NumberValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class DateFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  final int? minDate;
  final int? maxDate;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final DateValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;

  DateFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.defaultValue,
    this.pathToField,
    this.minDate,
    this.maxDate,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const DateValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class TimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final TimeValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.time;

  @override
  final double tableColumnWidth;

  TimeFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 100,
    this.validator = const TimeValidatorFF(),
  });
}

class EmailFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BaseEmailValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  EmailFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const EmailOrNullValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class PasswordFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;

  PasswordFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class PhoneNumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BasePhoneNumberValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  PhoneNumberFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const PhoneNumberOrNullValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class ExpandSelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final String optionsID;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  ExpandSelectFF({
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class ExpandMultiSelectFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final String optionsID;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;

  @override
  final List<String>? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final BaseStringListValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  ExpandMultiSelectFF({
    required this.displayName,
    required this.name,
    required this.optionsID,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator = const StringListOrNullValidatorFF(),
    this.tableColumnWidth = 150,
  });
}

class RadioGroupFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final List<dynamic> options;

  @override
  final List<String>? pathToField;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringValidatorFF validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  RadioGroupFF({
    required this.displayName,
    required this.name,
    required this.options,
    this.pathToField,
    this.defaultValue,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.validator = const StringValidatorFF(),
    this.tableColumnWidth = 100,
  });
}

class CheckboxFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final bool? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.bool;

  @override
  final double tableColumnWidth;
  CheckboxFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class CustomFFData {
  final String displayName;
  final String name;
  final BuildContext context;
  final Map<String, dynamic> data;
  final void Function(String name, dynamic value) onChanged;
  final FormFieldState<dynamic>? state;
  final List<FormItem> columns;

  CustomFFData({
    required this.displayName,
    required this.name,
    required this.context,
    required this.data,
    required this.onChanged,
    required this.columns,
    this.state,
  });
}

class CustomFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  final Widget Function(CustomFFData) builder;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  CustomFF({
    required this.displayName,
    required this.name,
    required this.builder,
    this.pathToField,
    this.showOnTable = true,
    this.dataBuilder,
    this.defaultValue,
    this.showOnForm = true,
    this.validator,
    this.tableColumnWidth = 170,
  });
}

class DateRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final dynamic defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;

  DateRangeFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startDate": DateValidatorFF(),
        "endDate": DateValidatorFF()
      },
    ),
  });
}

class DateTimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final dynamic defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;

  DateTimeRangeFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startDate": DateValidatorFF(),
        "endDate": DateValidatorFF()
      },
    ),
  });
}

class TimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final dynamic defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.timeRange;

  @override
  final double tableColumnWidth;

  TimeRangeFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 100,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "startTime": TimeValidatorFF(),
        "endTime": TimeValidatorFF()
      },
    ),
  });
}

class NumberRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final dynamic defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final MapOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.numberRange;

  @override
  final double tableColumnWidth;

  NumberRangeFF({
    required this.displayName,
    required this.name,
    this.pathToField,
    this.dataBuilder,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 130,
    this.validator = const MapOrNullValidatorFF(
      mapValidator: {
        "smallNumber": NumberValidatorFF(),
        "bigNumber": NumberValidatorFF()
      },
    ),
  });
}

class EmailListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final List<String>? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringListOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;

  EmailListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.tableColumnWidth = 150,
    this.validator,
  });
}

class PhoneNumberListFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final List<String>? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final StringListOrNullValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.stringList;

  @override
  final double tableColumnWidth;

  PhoneNumberListFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;

  ReadonlyDateFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyDateTimeRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.dateRange;

  @override
  final double tableColumnWidth;

  ReadonlyDateTimeRangeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyTimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;

  ReadonlyTimeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyNumberFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.number;

  @override
  final double tableColumnWidth;
  ReadonlyNumberFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ReadonlyTextFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;

  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.string;

  @override
  final double tableColumnWidth;

  ReadonlyTextFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateTimeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;

  ReadonlyDateTimeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ReadonlyDateRangeFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final int? defaultValue;
  @override
  final int Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.date;

  @override
  final double tableColumnWidth;

  ReadonlyDateRangeFF({
    required this.displayName,
    required this.name,
    this.dataBuilder,
    this.pathToField,
    this.showOnTable = true,
    this.showOnForm = true,
    this.defaultValue,
    this.validator,
    this.tableColumnWidth = 150,
  });
}

class ParagraphFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;

  ParagraphFF({
    required this.displayName,
    this.name = "",
    this.pathToField,
    this.showOnTable = false,
    this.showOnForm = true,
    this.defaultValue,
    this.dataBuilder,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class ParagraphHeaderFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;

  @override
  final List<String>? pathToField;
  @override
  final String? defaultValue;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;

  ParagraphHeaderFF({
    required this.displayName,
    this.name = "",
    this.pathToField,
    this.showOnTable = false,
    this.showOnForm = true,
    this.defaultValue,
    this.dataBuilder,
    this.validator,
    this.tableColumnWidth = 100,
  });
}

class NoneFF implements FormFF {
  @override
  final String displayName;
  @override
  final String name;
  @override
  final bool showOnTable;
  @override
  final bool showOnForm;
  @override
  final List<String>? pathToField;
  @override
  final String Function(Map<String, dynamic> rowData)? dataBuilder;
  @override
  final ValidatorFF? validator;
  @override
  final dynamic defaultValue;
  @override
  final FilterTypesFF filterType = FilterTypesFF.none;

  @override
  final double tableColumnWidth;
  NoneFF({
    this.displayName = "",
    this.name = "",
    this.pathToField,
    this.showOnTable = false,
    this.showOnForm = true,
    this.dataBuilder,
    this.validator,
    this.defaultValue,
    this.tableColumnWidth = 100,
  });
}
