import "package:flutter/material.dart";
import "package:group_radio_button/group_radio_button.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';

class RadioGroupFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final List<String> options;
  final String value;
  final Function(String name, String? value) onChanged;
  final FormFieldState<dynamic>? state;

  const RadioGroupFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.value,
    required this.onChanged,
    required this.options,
    this.state,
  }) : super(key: key);
  @override
  _RadioGroupFormState createState() => _RadioGroupFormState();
}

class _RadioGroupFormState extends State<RadioGroupFormFactory> {
  String _initilizedData = "";

  @override
  void initState() {
    super.initState();
    final data = DataClassUtils.dynamicToString(widget.value);
    _initilizedData = data;
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant RadioGroupFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_initilizedData != widget.value) {
      final data = DataClassUtils.dynamicToString(widget.value);
      _initilizedData = data;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: RadioGroup<String>.builder(
        direction: Axis.horizontal,
        groupValue: _initilizedData,
        onChanged: (value) {
          widget.onChanged(widget.name, value);
          widget.state?.didChange(value);
        },
        items: widget.options.map((e) => e.toString()).toList(),
        itemBuilder: (item) => RadioButtonBuilder(
          item,
        ),
      ),
    );
  }
}
