import "package:flutter/material.dart";
import 'package:verttice_logic/verttice_logic.dart';
import 'package:verttice_ui/src/widgets/border_label.dart';

enum ReadonlyFormType {
  number,
  text,
  time,
  date,
  dateTime,
  dateRange,
  dateTimeRange,
}

class ReadonlyFormFactory extends StatefulWidget {
  final String displayName;
  final String name;
  final dynamic value;
  final ReadonlyFormType type;
  const ReadonlyFormFactory({
    Key? key,
    required this.displayName,
    required this.name,
    required this.type,
    this.value,
  }) : super(key: key);

  @override
  _ReadonlyState createState() => _ReadonlyState();
}

class _ReadonlyState extends State<ReadonlyFormFactory> {
  dynamic _value;

  @override
  void initState() {
    super.initState();
    _value = widget.value;
  }

  @override
  void didUpdateWidget(covariant ReadonlyFormFactory oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (_value != widget.value) {
      _value = widget.value;
    }
  }

  @override
  Widget build(BuildContext context) {
    return BorderLabel(
      label: widget.displayName,
      child: Builder(
        builder: (BuildContext context) {
          return SelectableText(
            _value == null
                ? ""
                : widget.type == ReadonlyFormType.number
                    ? DataClassUtils.dynamicToDouble(_value)!.toStringAsFixed(2)
                    : widget.type == ReadonlyFormType.dateRange
                        ? _value == null || _value is! Map
                            ? ""
                            : "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(_value["startDate"], defultValue: 0)!, time: false) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(_value["endDate"], defultValue: 0)!, time: false) : ""}"
                        : widget.type == ReadonlyFormType.dateTimeRange
                            ? "${_value["startDate"] != null && _value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(_value["startDate"], defultValue: 0)!) : ""} - ${_value["endDate"] != null && _value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(_value["endDate"], defultValue: 0)!) : ""}"
                            : widget.type == ReadonlyFormType.text
                                ? (_value?.toString() ?? "")
                                : Dates.formatDataAndTime(
                                    miliseconds: DataClassUtils.dynamicToInt(
                                        _value ?? 0)!,
                                    date:
                                        (widget.type == ReadonlyFormType.date ||
                                                widget.type ==
                                                    ReadonlyFormType.dateTime)
                                            ? true
                                            : false,
                                    time:
                                        (widget.type == ReadonlyFormType.time ||
                                                widget.type ==
                                                    ReadonlyFormType.dateTime)
                                            ? true
                                            : false,
                                  ),
          );
        },
      ),
    );
  }
}
