import 'package:flutter/material.dart';
import 'package:html_editor_enhanced/html_editor.dart';
import 'package:verttice_ui/src/const/icons.dart';

class ThemedHtmlEditor extends StatefulWidget {
  final String? value;
  final void Function({required String fileContent}) fileContentCallback;
  const ThemedHtmlEditor({
    Key? key,
    this.value,
    required this.fileContentCallback,
  }) : super(key: key);

  @override
  State<ThemedHtmlEditor> createState() => _ThemedHtmlEditorState();
}

class _ThemedHtmlEditorState extends State<ThemedHtmlEditor> {
  String _value = '';
  @override
  void initState() {
    super.initState();
    if (widget.value != null) {
      _value = widget.value!;
    }
  }

  @override
  void didUpdateWidget(covariant ThemedHtmlEditor oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.value != _value) {
      _value = widget.value == null ? "" : widget.value!;
    }
  }

  final HtmlEditorController _richTextController = HtmlEditorController();

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButton: FloatingActionButton(
        onPressed: () async {
          final text = await _richTextController.getText();

          widget.fileContentCallback(fileContent: text);
          Navigator.pop(context);
        },
        child: ThemedIcons.check(),
      ),
      body: HtmlEditor(
        controller: _richTextController,
        htmlEditorOptions: HtmlEditorOptions(
          hint: "Your text here",
          initialText: _value,
        ),
        htmlToolbarOptions: const HtmlToolbarOptions(
          defaultToolbarButtons: [
            StyleButtons(),
            FontSettingButtons(fontSizeUnit: true),
            FontButtons(clearAll: true),
            ColorButtons(),
            ListButtons(listStyles: true),
            ParagraphButtons(
              textDirection: false,
              lineHeight: true,
              caseConverter: true,
            ),
            InsertButtons(
              video: true,
              audio: true,
              table: true,
              hr: true,
              otherFile: true,
              picture: true,
            ),
          ],
        ),
        // otherOptions: const OtherOptions(
        //   height: 400,
        // ),
      ),
    );
  }
}
